# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your option) any later
# version.
# 
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
# 
# You should have received a copy of the GNU Lesser General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 59 Temple
# Place - Suite 330, Boston, MA 02111-1307, USA, or go to
# http://www.gnu.org/copyleft/lesser.txt.
#-----------------------------------------------------------------------------
# Name        	: rct3.rb
# Description 	: Various stuff for creating custom scenery for Ataris RollerCoaster Tycoon 3
# Menu Item   	: Plugins\RCT3
# Author      	: Belgabor
# Usage     	: Copy script to PLUGINS folder in SketchUp folder
# Note	: This script is written to export ASE for importing custom scenery into Atari's RollerCoaster Tycoon 3, so there are lots of features missing.
#		  (And other seemingly strange ones present :p)
# Date        	: 12/12/2006
# Type        	: Various
# Version     	: 0.2 Improvements for SU6
#		  0.1 Initial Version
#-----------------------------------------------------------------------------

require 'sketchup.rb'
require 'parametric.rb'

def make_grid(gr, x, y, point)
	model = Sketchup.active_model
	model.start_operation "Create construction lines for #{gr}"
	
	entities = model.active_entities
	group = entities.add_group
	group.name = gr
	entities = group.entities
	
	entities.add_cline([x-2.m, y-2.m, 0], [x+2.m, y-2.m,0], "_")
	entities.add_cline([x-2.m, y+2.m, 0], [x+2.m, y+2.m,0], "_")
	entities.add_cline([x-2.m, y-2.m, 0], [x-2.m, y+2.m,0], "_")
	entities.add_cline([x+2.m, y-2.m, 0], [x+2.m, y+2.m,0], "_")
	
	entities.add_cline([x-2.m, y-1.m, 0], [x+2.m, y-1.m,0], ".")
	entities.add_cline([x-2.m, y+1.m, 0], [x+2.m, y+1.m,0], ".")
	entities.add_cline([x-1.m, y-2.m, 0], [x-1.m, y+2.m,0], ".")
	entities.add_cline([x+1.m, y-2.m, 0], [x+1.m, y+2.m,0], ".")
		
	entities.add_cline([x-2.m, y, 0], [x+2.m, y,0], ".")
	entities.add_cline([x, y-2.m, 0], [x, y+2.m,0], ".")
	
	if point>0
		entities.add_cline([x, y+2.m, 0], [x+2.m, y,0], ".")
		if (point == 1) or (point == 3)
			entities.add_cline([x, y-2.m, 0], [x+2.m, y,0], ".")
		end
		if (point == 2) or (point == 3)
			entities.add_cline([x-2.m, y, 0], [x, y+2.m,0], ".")
		end
	end
	
	model.commit_operation
end

def cl_full_tile
	make_grid("Full Tile Placement", 0, 0, 1)
end

def cl_quarter_tile
	gr = "1/4 Tile Placement"
	
	model = Sketchup.active_model
	model.start_operation "Create construction lines for #{gr}"
	
	entities = model.active_entities
	group = entities.add_group
	group.name = gr
	entities = group.entities
		
	entities.add_cline([-1.m, -1.m, 0], [1.m, -1.m,0], "_")
	entities.add_cline([-1.m, 1.m, 0], [1.m, 1.m,0], "_")
	entities.add_cline([-1.m, -1.m, 0], [-1.m, 1.m,0], "_")
	entities.add_cline([1.m, -1.m, 0], [1.m, 1.m,0], "_")
		
	entities.add_cline([-1.m, 0, 0], [1.m, 0,0], ".")
	entities.add_cline([0, -1.m, 0], [0, 1.m,0], ".")
	
	entities.add_cline([0.m, -1.m, 0], [1.m, 0,0], ".")
	entities.add_cline([0.m, 1.m, 0], [1.m, 0,0], ".")
	
	model.commit_operation
end

def cl_half_tile
	gr = "1/2 Tile Placement"
	
	model = Sketchup.active_model
	model.start_operation "Create construction lines for #{gr}"
	
	entities = model.active_entities
	group = entities.add_group
	group.name = gr
	entities = group.entities
		
	entities.add_cline([-2.m, -1.m, 0], [2.m, -1.m,0], "_")
	entities.add_cline([-2.m, 1.m, 0], [2.m, 1.m,0], "_")
	entities.add_cline([-2.m, -1.m, 0], [-2.m, 1.m,0], "_")
	entities.add_cline([2.m, -1.m, 0], [2.m, 1.m,0], "_")
		
	entities.add_cline([-2.m, 0, 0], [2.m, 0,0], ".")
	entities.add_cline([-1.m, -1.m, 0], [-1.m, 1.m,0], ".")
	entities.add_cline([1.m, -1.m, 0], [1.m, 1.m,0], ".")
	
	entities.add_cline([1.m, -1.m, 0], [2.m, 0,0], ".")
	entities.add_cline([1.m, 1.m, 0], [2.m, 0,0], ".")
	
	model.commit_operation
end

class RCT3_Wall < Parametric

	def create_entities(data, container)
	
	    w = data["w"].to_l
	    d = data["d"].to_l
	    h = data["h"].to_l
	    
	    pts = [[-(w/2),0,0], [w/2,0,0], [w/2,-d,0], [-(w/2),-d,0], [-(w/2),0,0]]
	    base = container.add_face pts
	    h = -h if base.normal.dot(Z_AXIS) < 0.0
	    base.pushpull h
	    pt = container.add_cpoint([0,0,0])
	    pt.set_attribute("ASE", "no_export", true)
	end
	
	def default_parameters
	    defaults = {"w", 4.m, "d", 20.cm, "h", 3.m}
	    defaults
	end
	
	def translate_key(key)
	    prompt = key
	    case( key )
		when "w"
		    prompt = "Width"
		when "h"
		    prompt = "Height"
		when "d"
		    prompt = "Depth"
	    end
	    prompt
	end

end

class RCT3_Corner < Parametric

	def create_entities(data, container)
	
	    w = data["w"].to_l
	    w2 = data["w2"].to_l
	    d = data["d"].to_l
	    h = data["h"].to_l
	    
	    pts = [[2.m-w,0,0], [2.m,0,0], [2.m,-w2,0], [2.m-d,-w2,0], [2.m-d,-d,0], [2.m-w,-d,0], [2.m-w,0,0]]
	    base = container.add_face pts
	    h = -h if base.normal.dot(Z_AXIS) < 0.0
	    base.pushpull h
	    
	    pt = container.add_cpoint([0,0,0])
	    pt.set_attribute("ASE", "no_export", true)
	end
	
	def default_parameters
	    defaults = {"w", 4.m, "w2", 4.m, "d", 20.cm, "h", 3.m}
	    defaults
	end
	
	def translate_key(key)
	    prompt = key
	    case( key )
		when "w"
		    prompt = "Primary Width"
		when "w2"
		    prompt = "Secondary Width"
		when "h"
		    prompt = "Height"
		when "d"
		    prompt = "Depth"
	    end
	    prompt
	end

end

def cp_selected
	s = Sketchup.active_model.selection
	return true if (s.first.class == Sketchup::ConstructionPoint)
end

def cp_edit_name
	s = Sketchup.active_model.selection.first
	t = s.get_attribute("ASE", "effect_name")
	v = [t]
	p = ["Name"]
	r = UI.inputbox(p,v,"Edit Effect Point Name")
	s.set_attribute("ASE", "effect_name", r[0])
end

def text_selected
	s = Sketchup.active_model.selection
	return true if (s.first.class == Sketchup::Text)
end

def text_edit_point
	s = Sketchup.active_model.selection.first
	pt = s.point
	v = [pt.x, pt.y, pt.z]
	p = ["X","Y","Z"]
	r = UI.inputbox(p,v,"Edit Effect Point")
	pt.x = r[0]
	pt.y = r[1]
	pt.z = r[2]
	s.point = pt
end

class TextMoveTool
# Based on SketchUp's LineTool example

	def initialize(pt, ot)
	    @ent = pt
	    @oldtool = ot
	    @ip2 = nil
	    @xdown = 0
	    @ydown = 0
	end

	def activate
	    @ip2 = Sketchup::InputPoint.new
	    @ip = Sketchup::InputPoint.new
	    @drawn = false
	    
	    self.reset(nil)
	end

	def deactivate(view)
	    view.invalidate if @drawn
	end

	def onMouseMove(flags, x, y, view)
		@ip2.pick view, x, y
		view.tooltip = @ip2.tooltip if( @ip2.valid? )
		view.invalidate
		
		if( (x-@xdown).abs > 10 || (y-@ydown).abs > 10 )
		    @dragging = true
		end
	end

	def onLButtonDown(flags, x, y, view)
		if( @ip2.valid? )
		    self.create_geometry(@ip1.position, @ip2.position,view)
		    self.reset(view)
		end
		view.lock_inference
	end

	def onLButtonUp(flags, x, y, view)
	    if( @dragging && @ip2.valid? )
		@ent.point = @ip2.position
		self.reset(view)
		Sketchup.active_model.select_tool(@oldtool)
	    end
	end

	def onKeyDown(key, repeat, flags, view)
	    if( key == CONSTRAIN_MODIFIER_KEY && repeat == 1 )
		@shift_down_time = Time.now
		
		if( view.inference_locked? )
		    view.lock_inference
		elsif( @ip2.valid? )
		    view.lock_inference @ip2
		end
	    end
	end

	def onKeyUp(key, repeat, flags, view)
	    if( key == CONSTRAIN_MODIFIER_KEY &&
		view.inference_locked? &&
		(Time.now - @shift_down_time) > 0.5 )
		view.lock_inference
	    end
	end

	def draw(view)
		if( @ip2.valid? )
		    @ip2.draw(view) if( @ip2.display? )
		    
		    view.set_color_from_line(@ent.point, @ip2)
		    self.draw_geometry(@ent.point, @ip2.position, view)
		    @drawn = true
		end
	end

	def onCancel(flag, view)
	    self.reset(view)
	    Sketchup.active_model.select_tool(@oldtool)
	end

	def reset(view)
	    Sketchup::set_status_text($exStrings.GetString("Select new position"), SB_PROMPT)
	    
	    @ip2.clear
	    
	    if( view )
		view.tooltip = nil
		view.invalidate if @drawn
	    end
	    
	    @drawn = false
	    @dragging = false
	end

	def draw_geometry(pt1, pt2, view)
	    view.draw_line(pt1, pt2)
	end

end

def text_move_point
	s = Sketchup.active_model.selection.first
	Sketchup.active_model.select_tool(TextMoveTool.new(s, Sketchup.active_model.select_tool(nil)))
end

if( not file_loaded?("rct3.rb") )

	main_menu = UI.menu("Plugins").add_submenu("RCT3")
	placement = main_menu.add_submenu("Placement")
	placement.add_item("Full Tile") { make_grid("Full Tile Placement", 0, 0, 1) }
	placement.add_item("Quarter Tile") { cl_quarter_tile }
	placement.add_item("Half Tile") { cl_half_tile }
	placement.add_separator
	placement.add_item("Wall") { make_grid("Wall Placement", 0, -2.m, 2) }
	placement.add_item("Corner") { make_grid("Corner Placement", 0, -2.m, 3) }
	placement.add_separator
	placement.add_item("Path Edge (inner)") { make_grid("Path Edge (inner) Placement", 0, -140.cm, 2) }
	placement.add_item("Path Edge (outer, join)") { make_grid("Path Edge (outer, join) Placement", 0, -180.cm, 2) }
	placement.add_item("Path Center") { make_grid("Path Center Placement", 0, 0, 2) }
	
	objects_menu = main_menu.add_submenu("Objects")
	objects_menu.add_item("RCT3 Wall") { RCT3_Wall.new }
	objects_menu.add_item("RCT3 Corner") { RCT3_Corner.new }

	UI.add_context_menu_handler do |menu|
		if (cp_selected)
			menu.add_separator
			sm = menu.add_submenu("RCT3 ConstructionPoint Settings")
			sm.add_item("Edit Name") { cp_edit_name }
		end
		if (Sketchup.version_number >= 6000000 and text_selected)
			menu.add_separator
			sm = menu.add_submenu("RCT3 Text Settings")
			sm.add_item("Edit Point") { text_edit_point }
			sm.add_item("Move Point") { text_move_point }
		end
	end
end
file_loaded("rct3.rb")
